'''dossier.label.combined_store

.. This software is released under an MIT/X11 open source license.
   Copyright 2012-2015 Diffeo, Inc.
'''

from __future__ import absolute_import

from dossier.label.label import LabelStore
from dossier.label.relation_label import RelationLabelStore


class CombinedLabelStore(object):
    '''Class providing utilities that need both label stores at the same
    time.
    '''

    def __init__(self, kvl):
        self.kvl = kvl
        self.label_store = LabelStore(kvl)
        self.rel_label_store = RelationLabelStore(kvl)

    def get_related_coref_relationships(self, content_id, min_strength=None):
        '''Follow coreference relationships to get full related graph.

        :rtype: dict mapping related identifier to list of documents
        coref to that identifier.
        '''
        related_labels = self.rel_label_store.get_related(
            content_id, min_strength=min_strength)
        related_cids = [rel_label.other(content_id)
                        for rel_label in related_labels]
        related_cid_to_idents = {}
        for cid in related_cids:
            conn_labels = self.label_store.directly_connected(cid)
            idents = [label.other(cid) for label in conn_labels]
            related_cid_to_idents[cid] = idents
        return related_cid_to_idents

    def get_related_flat(self, content_id, min_strength=None):
        '''Follow coreference relationships to get full related graph.

        This differs from ``get_related_coref_relationships`` in that
        it returns a flat list of all identifiers found through the
        coreference layer of indirection.

        :rtype: list of identifiers
        '''
        rel_id_to_idents = self.get_related_coref_relationships(
            content_id, min_strength=min_strength)
        flat_list = []
        for val in rel_id_to_idents.values():
            flat_list.extend(val)
        return flat_list



'''dossier.label.tests

.. This software is released under an MIT/X11 open source license.
   Copyright 2012-2015 Diffeo, Inc.
'''
from __future__ import absolute_import

import pytest

from dossier.label.label import LabelStore, Label
from dossier.label.relation_label import RelationLabelStore, \
    RelationLabel, RelationStrength
from dossier.label.combined_store import CombinedLabelStore
from dossier.label.tests import kvl


@pytest.yield_fixture
def combined_label_store(kvl):
    store = CombinedLabelStore(kvl)
    yield store
    store.label_store.delete_all()
    store.rel_label_store.delete_all()


def test_relationship_graph(combined_label_store):
    entities = ['A', 'B', 'C', 'D', 'E', 'F']

    for ent in entities:
        coref_ents = [ent+str(i) for i in xrange(3)]
        for coref_ent in coref_ents:
            label = Label(ent, coref_ent, 'foo', 1)
            combined_label_store.label_store.put(label)

    rl1 = RelationLabel('A', 'B', 'foo', RelationStrength.STRONG)
    rl2 = RelationLabel('A', 'C', 'foo', RelationStrength.WEAK)
    rl3 = RelationLabel('A', 'D', 'foo', RelationStrength.NONE)
    rl4 = RelationLabel('A', 'E', 'foo', RelationStrength.AKA)
    rl5 = RelationLabel('A', 'F', 'foo', RelationStrength.NONE)
    combined_label_store.rel_label_store.put(
        rl1, rl2, rl3, rl4, rl5)

    cid_to_related = combined_label_store.get_related_coref_relationships('A')
    assert frozenset(cid_to_related['C']) == frozenset(['C0', 'C1', 'C2'])
    assert frozenset(cid_to_related['B']) == frozenset(['B0', 'B1', 'B2'])
    assert frozenset(cid_to_related['E']) == frozenset(['E0', 'E1', 'E2'])
